#include "AdjustTime.h"
#include <gtkmm/treeview.h>
#include "utility.h"

/*
 *
 */
bool AdjustTime::execute(Document &doc, 
		const long &time_msecs, bool to_start, bool to_duration)
{
	Glib::RefPtr<Gtk::TreeSelection> selection = doc.get_subtitle_view()->get_selection();

	std::vector<Gtk::TreeModel::Path> rows = selection->get_selected_rows();

	if(rows.size() == 0)
	{
		doc.flash_message(_("Please select at least a subtitle."));
		return false;
	}


	if(to_start && !to_duration)
	{
		for(unsigned int i=0; i<rows.size(); ++i)
		{
			SubtitleModifier subtitle(doc.get_subtitle_model()->get_iter(rows[i]));
			//
			subtitle.set_start( 
					SubtitleTime( subtitle.get_start().totalmsecs + time_msecs));
		}
	}
	else if(!to_start && to_duration)
	{
		for(unsigned int i=0; i<rows.size(); ++i)
		{
			SubtitleModifier subtitle(doc.get_subtitle_model()->get_iter(rows[i]));
			//
			subtitle.set_end( 
					SubtitleTime( subtitle.get_end().totalmsecs + time_msecs));
		}
	}
	else if(to_start && to_duration)
	{
		for(unsigned int i=0; i<rows.size(); ++i)
		{
			SubtitleModifier subtitle(doc.get_subtitle_model()->get_iter(rows[i]));
			//
			subtitle.set_start_and_end( 
					SubtitleTime( subtitle.get_start().totalmsecs + time_msecs),
					SubtitleTime( subtitle.get_end().totalmsecs + time_msecs));
		}
	}

	doc.emit_signal("subtitle-time-changed");

	return true;
}

/*
 *	"very-short"
 *	"short"
 *	"medium"
 *	"long"
 */
bool AdjustTime::execute(Document &doc, AdjustTime::TYPE type, const Glib::ustring &size, bool to_start, bool to_duration)
{
	int value = 0;

	if(size == "very-short")
		Config::getInstance().get_value_int("adjust-time", "very-short", value, SubtitleTime(0,0,0,100).totalmsecs);
	else if(size == "short")
		Config::getInstance().get_value_int("adjust-time", "short", value, SubtitleTime(0,0,0,500).totalmsecs);
	else if(size == "medium")
		Config::getInstance().get_value_int("adjust-time", "medium", value, SubtitleTime(0,0,1,000).totalmsecs);
	else if(size == "long")
		Config::getInstance().get_value_int("adjust-time", "long", value, SubtitleTime(0,0,5,000).totalmsecs);

	if(type == REMOVE)
		value = -value;

	return execute(doc, value, to_start, to_duration);
}
