/*
 *	subtitle editor
 *
 *	http://kitone.free.fr/subtitleeditor/
 *
 *	Copyright @ 2005-2006, kitone
 *
 *	Contact: kitone at free dot fr
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public
 *	License as published by the Free Software Foundation; either
 *	version 2 of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU
 *	General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public
 *	License along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA	02111-1307	USA
 *
 *	See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *	\file
 *	\brief 
 *	\author kitone (kitone at free dot fr)
 */

#include "MenuBar.h"
#include "utility.h"
#include <gtkmm/accelmap.h>
#include "DocumentSystem.h"
#include "Config.h"

#include "ActionSystem.h"
#include "gui/PreferencesUI.h"


/*
 *
 */
MenuBar::MenuBar()
:Gtk::VBox(false, 0), m_statusbar(NULL)
{
}

/*
 *
 */
void MenuBar::connect_proxy(const Glib::RefPtr<Gtk::Action> &action, Gtk::Widget *widget)
{
	if(Gtk::MenuItem *item = dynamic_cast<Gtk::MenuItem*>(widget))
	{
		Glib::ustring tooltip = action->property_tooltip();

		item->signal_select().connect(
				sigc::bind(
					sigc::mem_fun(m_statusbar, &Statusbar::push_text), tooltip));
		item->signal_deselect().connect(
				sigc::mem_fun(m_statusbar, &Statusbar::pop_text));
	}
}

/*
 *
 */
void MenuBar::create(Gtk::Window &window, Statusbar &statusbar)
{
	m_statusbar = &statusbar;
	/*
	struct MenuItem
	{
		gchar *action_name;
		gchar *stock_id;
		gchar *label;
		gchar *tooltip;
		gchar *accel;
	};

	MenuItem menuitems[]={
		{"menu-file", Gtk::Stock::NEW, NULL, _("Create a new document"), NULL},
		{"open-subtitle", NULL, NULL, _("Open a file"), NULL}
		};
	*/
	m_refActionGroup = Gtk::ActionGroup::create();

	// menu-file
	addMenuItem("menu-file", _("_File"));

	addActionItem("new-subtitle", Gtk::Stock::NEW, 
			"", _("Create a new document"));
	addActionItem("open-subtitle", Gtk::Stock::OPEN, 
			"", _("Open a file"));
	addActionItem("save-subtitle", Gtk::Stock::SAVE, 
			"", _("Save the current file"));
	addActionItem("save-as-subtitle", Gtk::Stock::SAVE_AS, 
			"", _("Save the current file with a different name"));
	addActionItem("properties", _("_Properties"),
			"");

	addMenuItem("menu-translation", _("Translation"));
	addActionItem("open-translation", _("Open Translation"), _("Open translation from file."));
	addActionItem("save-translation",	_("Save Translation"),	_("Save translation to file."));

	addMenuItem("menu-import", _("Import"));
	addActionItem("import-text", _("Text"), 
			_("Create a new subtitle with any text file."));

	addMenuItem("menu-export", _("Export"));
	addActionItem("export-text", _("Text"), _("Export just a text."));

	addActionItem("close-subtitle", Gtk::Stock::CLOSE, 
			"", _("Close the current document"));
	addActionItem("quit", Gtk::Stock::QUIT, 
			_("Exit"), _("Quit the program"));

	
	// menu-edit
	addMenuItem("menu-edit", _("_Edit"));
	
	addActionItem("select-previous-subtitle", Gtk::Stock::GO_BACK, 
			_("Select Previous Subtitle"), _("Select the previous subtitle"));
	addActionItem("select-next-subtitle", Gtk::Stock::GO_FORWARD, 
			_("Select Next Subtitle"), _("Select the next subtitle"));

	addActionItem("delete-subtitle", Gtk::Stock::DELETE, _("Delete"), 
			_("Delete the selected subtitles"), "<Control>Delete");
	addActionItem("insert-before", Gtk::Stock::GOTO_BOTTOM, 
			_("Insert Before"), _("Insert blank subtitle before the selected subtitle"), "<Control>Insert"); 
	addActionItem("insert-after", Gtk::Stock::GOTO_TOP, 
			_("Insert After"), _("Insert blank subtitle after the selected subtitle"), "Insert");
	
	// TODO: remove me
	//addMenuItem("menu-move", _("_Move"));

	addActionItem("move-subtitles", Gtk::Stock::JUMP_TO, _("_Move Subtitles"), 
			_("All subtitles will be also moved after this subtitle"), "<Control>M");

	addActionItem("add-100-ms", Gtk::Stock::ADD, _("Add 100 Milliseconds"), 
			_("Add 100 Milliseconds to all subtitles selected"));
	addActionItem("remove-100-ms", Gtk::Stock::REMOVE, _("Remove 100 Milliseconds"),
			_("Remove 100 Milliseconds to all subtitles selected"));

	addActionItem("add-100-ms-to-start", Gtk::Stock::ADD, _("Add 100 Milliseconds To Start"),
			_("Add 100 Milliseconds to start for all subtitles selected"));
	addActionItem("remove-100-ms-to-start", Gtk::Stock::REMOVE, _("Remove 100 Milliseconds To Start"),
			_("Remove 100 Milliseconds to start for all subtitles selected"));

	addActionItem("add-100-ms-to-duration", Gtk::Stock::ADD, _("Add 100 Milliseconds To Duration"),
			_("Add 100 Milliseconds to duration for all subtitles selected"));
	addActionItem("remove-100-ms-to-duration", Gtk::Stock::REMOVE, _("Remove 100 Milliseconds To Duration"),
			_("Remove 100 Milliseconds to duration for all subtitles selected"));

	addActionItem("apply-translation", Gtk::Stock::APPLY, _("Apply Translation"),
			_("Replace the text of the subtitle by the translation"));
	
	addActionItem("split-subtitle", _("Split Subtitle"),
			_("Split the selected subtitles"));
	addActionItem("merge-subtitles", _("Merge Subtitles"),
			_("Merge the selected subtitles"));

	addMenuItem("menu-move-forward", Gtk::Stock::ADD, _("Move Forward"));
	addActionItem("very-short-forward-move", _("Very Short Forward Move"));
	addActionItem("short-forward-move", _("Short Forward Move"));
	addActionItem("medium-forward-move", _("Medium Forward Move"));
	addActionItem("long-forward-move", _("Long Forward Move"));

	addMenuItem("menu-move-backwards", Gtk::Stock::REMOVE, _("Move Backwards"));
	addActionItem("very-short-backwards-move", _("Very Short Backwards Move"));
	addActionItem("short-backwards-move", _("Short Backwards Move"));
	addActionItem("medium-backwards-move", _("Medium Backwards Move"));
	addActionItem("long-backwards-move", _("Long Backwards Move"));
	
	// menu-tools
	addMenuItem("menu-tools", _("_Tools"));
	
	addActionItem("find-and-replace", Gtk::Stock::FIND_AND_REPLACE, _("_Find And Replace"), 
			_("Search for and replace text"), "<Control>F");
	addActionItem("remove-line-empty", _("Remove Line Empty"));
	addActionItem("check-errors", Gtk::Stock::YES, _("Check Errors"),
			_("Launch the errors checking"));
	addActionItem("spell-check", Gtk::Stock::SPELL_CHECK, _("Spell Check"),
			_("Launch the spell checking"));
	addActionItem("set-all-end-time", _("Set All End Time"));
	addActionItem("style-editor", _("Style Editor"),
			_("Launch the style editor"));
	addActionItem("split-document", Gtk::Stock::CUT, _("Split Document"),
			_("Split the current document in two"));
	addActionItem("join-document", Gtk::Stock::CONNECT, _("Join Document"), 
			_("Add subtitles from file"));
	addActionItem("change-framerate", Gtk::Stock::CONVERT, _("Change Framerate"), 
			_("Convert framerate"));
	addActionItem("scale", Gtk::Stock::CONVERT, _("Scale"), 
			_("Scale by two point"));

	// menu-video
	addMenuItem("menu-video", _("Vide_o"));
	
	addActionItem("video-player/open", Gtk::Stock::OPEN, _("Open Movie"), 
			_("Open the video"), "<Shift><Control>o");
	addActionItem("video-player/play", Gtk::Stock::MEDIA_PLAY,
			_("Play the video"));
	addActionItem("video-player/pause", Gtk::Stock::MEDIA_PAUSE, 
			_("Make a pause"));
	addActionItem("video-player/play-pause", Gtk::Stock::MEDIA_PLAY, 
			_("Play / Pause"), _("Play or make a pause"));

	addActionItem("video-player/seek-to-selection", _("Seek To Selection"),
			_("Seek to the first selected subtitle"));

	addActionItem("video-player/play-previous-subtitle", Gtk::Stock::MEDIA_PREVIOUS, _("Play Previous Subtitle"), 
			_("Play previous subtitle from the first selected subtitle"));
	addActionItem("video-player/play-current-subtitle", Gtk::Stock::MEDIA_PLAY, _("Play Selection"), 
			_("Play the selected subtitle"));
	addActionItem("video-player/play-next-subtitle", Gtk::Stock::MEDIA_NEXT, _("Play Next Subtitle"), 
			_("Play next subtitle from the first selected subtitle"));
	
	addActionItem("video-player/play-previous-second", _("Play Previous Second"));
	addActionItem("video-player/play-first-second", _("Play First Second"));
	addActionItem("video-player/play-last-second", _("Play Last Second"));
	addActionItem("video-player/play-next-second", _("Play Next Second"));

	addActionItem("video-player/set-subtitle-start", _("Set Subtitle _Start"));
	addActionItem("video-player/set-subtitle-end", _("Set Subtitle _End"));

	addMenuItem("video-player/jump", _("Jump"));
	addActionItem("video-player/very-short-backwards-jump", _("Very Short Backwards Jump"));
	addActionItem("video-player/very-short-forward-jump", _("Very Short Forward Jump"));
	addActionItem("video-player/short-backwards-jump", _("Short Backwards Jump"));
	addActionItem("video-player/short-forward-jump", _("Short Forward Jump"));
	addActionItem("video-player/medium-backwards-jump", _("Medium Backwards Jump"));
	addActionItem("video-player/medium-forward-jump", _("Medium Forward Jump"));
	addActionItem("video-player/long-backwards-jump", _("Long Backwards Jump"));
	addActionItem("video-player/long-forward-jump", _("Long Forward Jump"));

	// menu-waveform
	addMenuItem("menu-waveform", _("_Waveform"));
	
	addActionItem("waveform/open", Gtk::Stock::OPEN, _("Open Waveform"));
	addActionItem("waveform/save", Gtk::Stock::SAVE, _("Save Waveform"));

	addActionItem("waveform/zoom-in", Gtk::Stock::ZOOM_IN, _("Zoom In"));
	addActionItem("waveform/zoom-out", Gtk::Stock::ZOOM_OUT, _("Zoom Out"));
	addActionItem("waveform/zoom-selection", Gtk::Stock::ZOOM_FIT, _("Zoom Selection"));
	addActionItem("waveform/zoom-all", Gtk::Stock::ZOOM_100, _("Zoom All"));

	addActionItem("waveform/center-with-selected-subtitle", _("Center With Selected Subtitle"));

	addToggleAction("waveform/scrolling-with-cursor", _("Scrolling With Cursor"), "waveform", "scrolling-with-cursor");
	addToggleAction("waveform/scrolling-with-selection", _("Scrolling With Selection"), "waveform", "scrolling-with-selection");


	// menu-view
	addMenuItem("menu-view", _("_View"));

	addActionItem("view/simple", _("Simple"));
	addActionItem("view/advanced", _("Advanced"));
	addActionItem("view/translation", _("Translation"));
	addActionItem("view/timing", _("Timing"));

	addToggleAction("display-video-player", _("Video Player"), "interface", "display-video-player");
	addToggleAction("display-waveform", _("Waveform"), "interface", "display-waveform");

	addMenuItem("menu-columns", _("Columns"));

	addToggleActionColumn("number", _("Number"));
	addToggleActionColumn("layer", _("Layer"));
	addToggleActionColumn("start", _("Start"));
	addToggleActionColumn("end", _("End"));
	addToggleActionColumn("duration", _("Duration"));
	addToggleActionColumn("style", _("Style"));
	addToggleActionColumn("name", _("Name"));
	addToggleActionColumn("margin-r", _("Margin Left"));
	addToggleActionColumn("margin-l", _("Margin Right"));
	addToggleActionColumn("margin-v", _("Margin Vertical"));
	addToggleActionColumn("effect", _("Effect"));
	addToggleActionColumn("text", _("Text"));
	addToggleActionColumn("translation", _("Translation"));
	//addToggleActionColumn("characters-per-line", _("Characters Per Line"));
	addToggleActionColumn("note", _("Note"));

	// menu-external
	addMenuItem("menu-preview", _("Preview"));
	addActionItem("preview/open", Gtk::Stock::OPEN, 
			_("Open Movie"), _("Open Movie with external video player."), "");
	addActionItem("preview/play", Gtk::Stock::MEDIA_PLAY, 
			_("Play Movie"), _("Play Movie with external video player."), "<Control>space");

	// menu-option
	addMenuItem("menu-option", _("_Options"));
	
	addActionItem("preferences", Gtk::Stock::PREFERENCES, "",
			_("Configure Subtitle Editor"));

	// menu-help
	addMenuItem("menu-help", _("_Help"));
	addActionItem("about", Gtk::Stock::HELP, _("_About"));


	show_all();

	// UIManager
	m_refUIManager = Gtk::UIManager::create();
	m_refUIManager->signal_connect_proxy().connect(
			sigc::mem_fun(*this, &MenuBar::connect_proxy));

	m_refUIManager->insert_action_group(m_refActionGroup);

	window.add_accel_group(m_refUIManager->get_accel_group());

	m_refUIManager->add_ui_from_file(get_share_dir("menubar.xml"));

	pack_start(*m_refUIManager->get_widget("/menubar"), false, false);

	show_all();

	Config::getInstance().signal_changed("subtitle-view").connect(
			sigc::mem_fun(*this, &MenuBar::on_config_subtitle_view_changed));

	ActionSystem::getInstance().signal_sensitive_changed().connect(
			sigc::mem_fun(*this, &MenuBar::set_sensitive));

#define DISABLE(name) m_refActionGroup->get_action(name)->set_sensitive(false)
	DISABLE("properties");
	DISABLE("split-subtitle");
	DISABLE("merge-subtitles");
	DISABLE("remove-line-empty");
	DISABLE("set-all-end-time");
#undef DISABLE
}

/*
 *
 */
MenuBar::~MenuBar()
{
}


/*
 *
 */
void MenuBar::addActionItem(
				const Glib::ustring &name, 
				const Gtk::StockID &stock_id, 
				const Glib::ustring &label,
				const Glib::ustring &tooltip,
				const Glib::ustring &accel
				// callback
		)
{
	if(accel.empty())
	{
		m_refActionGroup->add(Gtk::Action::create(name, stock_id, label, tooltip), 
				sigc::bind<Glib::ustring>(sigc::mem_fun(this, &MenuBar::execute), name));
	}
	else
	{
		m_refActionGroup->add(Gtk::Action::create(name, stock_id, label, tooltip), Gtk::AccelKey(accel), 
				sigc::bind<Glib::ustring>(sigc::mem_fun(this, &MenuBar::execute), name));
	}
}

/*
 *
 */
void MenuBar::addActionItem(
				const Glib::ustring &name, 
				const Glib::ustring &label,
				const Glib::ustring &tooltip,
				const Glib::ustring &accel
				// callback
		)
{
	if(accel.empty())
	{
		m_refActionGroup->add(Gtk::Action::create(name, label, tooltip),
				sigc::bind<Glib::ustring>(sigc::mem_fun(this, &MenuBar::execute), name));
	}
	else
	{
		m_refActionGroup->add(Gtk::Action::create(name, label, tooltip), Gtk::AccelKey(accel),
				sigc::bind<Glib::ustring>(sigc::mem_fun(this, &MenuBar::execute), name));
	}
}

/*
 *
 */
void MenuBar::addToggleActionColumn(const Glib::ustring &name, const Glib::ustring &label	)
{
	bool default_value = false;

	Glib::RefPtr<Gtk::Action> action = Gtk::ToggleAction::create(name, label);
	Glib::RefPtr<Gtk::ToggleAction> toggle_action = Glib::RefPtr<Gtk::ToggleAction>::cast_static(action);

	if(Config::getInstance().get_value_bool("subtitle-view", Glib::ustring("show-column-") + name, default_value))
		toggle_action->set_active(default_value);

	sigc::slot<void> slot = sigc::bind<Glib::ustring>(
			sigc::mem_fun(*this, &MenuBar::on_setup_view_toggled), name);

	m_connections[name] = action->signal_activate().connect(slot);

	m_refActionGroup->add(action);
}

/*
 *
 */
void MenuBar::addToggleAction(const Glib::ustring &name, const Glib::ustring &label, const Glib::ustring &group, const Glib::ustring &key )
{
	Glib::RefPtr<Gtk::Action> action = Gtk::ToggleAction::create(name, label);
	Glib::RefPtr<Gtk::ToggleAction> toggle_action = Glib::RefPtr<Gtk::ToggleAction>::cast_static(action);

	bool value = false;
	if(Config::getInstance().get_value_bool(group, key, value))
		toggle_action->set_active(value);

	/*m_connections[name] = */action->signal_activate().connect(
				sigc::bind<Glib::ustring>(sigc::mem_fun(this, &MenuBar::execute), name));

	m_refActionGroup->add(action);
}
/*
 *
 */
void MenuBar::addMenuItem(	const Glib::ustring &name, const Glib::ustring &label)
{
	m_refActionGroup->add(Gtk::Action::create(name, label));
}

/*
 *
 */
void MenuBar::addMenuItem(	const Glib::ustring &name, 
														const Gtk::StockID & stock_id, 
														const Glib::ustring &label)
{
	m_refActionGroup->add(Gtk::Action::create(name, stock_id, label));
}

/*
 *
 */
void MenuBar::execute(const Glib::ustring &name)
{
	//std::cout << name << std::endl;

	ActionSystem::getInstance().execute(name);

	if(name == "preferences")
		on_preferences();
	else if(name == "display-video-player")
	{
		Glib::RefPtr<Gtk::ToggleAction> action = 
			Glib::RefPtr<Gtk::ToggleAction>::cast_static(m_refActionGroup->get_action(name));

		bool value = action->get_active();
		Config::getInstance().set_value_bool("interface", "display-video-player", value);

	}
	else if(name == "display-waveform")
	{
		Glib::RefPtr<Gtk::ToggleAction> action = 
			Glib::RefPtr<Gtk::ToggleAction>::cast_static(m_refActionGroup->get_action(name));

		bool value = action->get_active();
		
		Config::getInstance().set_value_bool("interface", "display-waveform", value);
	}
	else if(name == "waveform/scrolling-with-cursor")
	{
		Glib::RefPtr<Gtk::ToggleAction> action = 
			Glib::RefPtr<Gtk::ToggleAction>::cast_static(m_refActionGroup->get_action(name));

		bool value = action->get_active();

		Config::getInstance().set_value_bool("waveform", "scrolling-with-cursor", value);
	}
	else if(name == "waveform/scrolling-with-selection")
	{
		Glib::RefPtr<Gtk::ToggleAction> action = 
			Glib::RefPtr<Gtk::ToggleAction>::cast_static(m_refActionGroup->get_action(name));

		bool value = action->get_active();

		Config::getInstance().set_value_bool("waveform", "scrolling-with-selection", value);
	}
}


// FILE

/*
 *
 */
void MenuBar::on_setup_view_toggled(const Glib::ustring &column)
{
	Glib::RefPtr<Gtk::ToggleAction> action = 
		Glib::RefPtr<Gtk::ToggleAction>::cast_static(m_refActionGroup->get_action(column));

	if(action)
	{
		sigc::connection connection = m_connections[column];

		bool state = action->get_active();
	
		connection.block();
	
		Config::getInstance().set_value_bool("subtitle-view", Glib::ustring("show-column-")+column, state);
		
		connection.unblock();
	}
}

/*
 *
 */
void MenuBar::set_toggle_column(const Glib::ustring &column, bool state)
{
	try
	{
		Glib::RefPtr<Gtk::ToggleAction> action = 
			Glib::RefPtr<Gtk::ToggleAction>::cast_static(m_refActionGroup->get_action(column));

		if(action)
		{
			/*
			connection.block();
			
			action->set_active(state);
			
			connection.unbblock();
			*/
			Config::getInstance().set_value_bool("subtitle-view", Glib::ustring("show-column-")+column, state);
		}
	}
	catch(Glib::Error &ex)
	{
		std::cerr << ex.what() << std::endl;
	}
}

/*
 *
 */
void MenuBar::on_preferences()
{
	PreferencesUI*	dialog = NULL;
	Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(get_share_dir("dialog-preferences.glade"), "dialog-preferences");
	refXml->get_widget_derived("dialog-preferences", dialog);

	dialog->run();

	delete dialog;
}

/*
 *
 */
void MenuBar::on_config_subtitle_view_changed(const Glib::ustring &key, const Glib::ustring &value)
{
	std::cout << build_message("%s %s %s %s", __FILE__, __FUNCTION__, key.c_str(), value.c_str()) << std::endl;
	
	if(key.find("show-column-") != Glib::ustring::npos)
	{
		Glib::ustring column = key;
		column.erase(0, Glib::ustring("show-column-").size());

		Glib::RefPtr<Gtk::ToggleAction> action = 
			Glib::RefPtr<Gtk::ToggleAction>::cast_static(m_refActionGroup->get_action(column));

		bool state;
		if(from_string(value, state))
		{
			m_connections[column].block();
			action->set_active(state);
			m_connections[column].unblock();
		}
	}
}

/*
 *
 */
void MenuBar::set_sensitive(const Glib::ustring &name, bool state)
{
	Glib::RefPtr<Gtk::Action> action = m_refActionGroup->get_action(name);
	action->set_sensitive(state);
}

