#include <iostream>
#include <config.h>
#include <gtkmm/main.h>
#include "gui/Application.h"

#include "debug.h"
#include "DocumentSystem.h"
#include "utility.h"

#include <ctime>
#include <gst/gst.h>
#include <gdk/gdkx.h>
#include <glib.h>
#include <gtkmm/socket.h>

#include "gui/MPlayer.h"
#include "gui/GStreamerPlayer.h"
#include "SubtitleSystem.h"
#include "Options.h"

#ifdef ENABLE_UNITTEST
	bool launch_unittest();
#endif


/*
 *
 */
void check_config_dir()
{
	Glib::ustring path = Glib::build_filename(Glib::get_home_dir(), ".subtitleeditor");
	if(Glib::file_test(path, Glib::FILE_TEST_IS_DIR) == false)
	{
		//g_mkdir(path.c_str(), 0700);
		Glib::spawn_command_line_async("mkdir " + path);
	}
}

#ifdef DEBUG

void parse_debug_options(OptionGroup &op)
{
	int flags = 0;
	if(op.debug_all)
	{
		flags |= SE_DEBUG_ALL;

		se_debug_init(flags);
	}
	else
	{
		if(op.debug_app)
			flags |= SE_DEBUG_APP;
		if(op.debug_view)
			flags |= SE_DEBUG_VIEW;
		if(op.debug_loader)
			flags |= SE_DEBUG_LOADER;
		if(op.debug_saver)
			flags |= SE_DEBUG_SAVER;
		if(op.debug_search)
			flags |= SE_DEBUG_SEARCH;
		if(op.debug_regex)
			flags |= SE_DEBUG_REGEX;
		if(op.debug_video_player)
			flags |= SE_DEBUG_VIDEO_PLAYER;
		if(op.debug_spell_checking)
			flags |= SE_DEBUG_SPELL_CHECKING;
		if(op.debug_waveform)
			flags |= SE_DEBUG_WAVEFORM;
		if(op.debug_utility)
			flags |= SE_DEBUG_UTILITY;

		se_debug_init(flags);
	}
}

#endif//DEBUG

/*
 *
 */
int main(int argc, char *argv[])
{
	if(!g_thread_supported())
		g_thread_init (NULL);

	// DEBUG
	std::clock_t start = std::clock();

	// Bindtextdomain
	bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain(PACKAGE);

 
	// init Gtk+
	Gtk::Main kit(argc, argv);

	Glib::set_application_name("subtitleeditor");

	check_config_dir();

	// SubtitleEditor Options
	OptionGroup options;
	try
	{
		Glib::OptionContext context(_(" - edit subtitles files"));
		context.set_main_group(options);
		
		Glib::OptionGroup gst_group(gst_init_get_option_group());
		context.add_group(gst_group);
		
		Glib::OptionGroup gtk_group(gtk_get_option_group(TRUE));
		context.add_group(gtk_group);

		//context.set_ignore_unknown_options(true);
		context.parse(argc, argv);
	}
	catch(const Glib::Error &ex)
	{
		std::cerr << "Error loading options : " << ex.what() << std::endl;
	}
	
	// Init Debug
#ifdef DEBUG
	parse_debug_options(options);
	se_debug_message(SE_DEBUG_APP, "Startup subtitle version %s", VERSION);
#endif//DEBUG

	// Init GStreamer 
	se_debug_message(SE_DEBUG_APP, "Init GStreamer");
	
	gst_init(&argc, &argv);

#ifdef ENABLE_UNITTEST
	if(options.launch_unittest)
	{
		return launch_unittest();
	}
#endif//ENABLE_UNITTEST
	
	// Run Application
	//Application app(argc, argv);
	Application*	application = NULL;
	Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(get_share_dir("subtitleeditor.glade"), "window-main");
	refXml->get_widget_derived("window-main", application);
	refXml.clear();

#warning "FIXME: 1.0s sans app.show(), 0.58s avec app.show()"
	application->show();

	application->init(options);

	//application->show();

	// DEBUG
	std::cout << "application run: " << double(std::clock() - start) / CLOCKS_PER_SEC << std::endl;
	
	kit.run(*application);

	delete application;

	return EXIT_SUCCESS;
}
